package algos.introduction

import java.io._
import algos.random.RandomPermute

object StableMatchingGen {

  def main(args : Array[String]) {
    try {
      if (args.length != 2) throw new ArgsError
      val n = args(0).toInt
      val instance = generateInstance(n)
      write(instance, args(1))
    }
    catch {
      case e : Throwable => {
        println("ERROR: " + e.getMessage())
        usage()
        sys.exit(-1)
      }
    }
  }

  private def usage() {
    println("USAGE: scala StableMatchingGen.jar <n> <file name>");
  }

  private def write(instance : Array[Array[Int]], fileName : String) {
    val f = new PrintWriter(new BufferedWriter(new FileWriter(fileName)))
    try {
      for (plist <- instance) {
        for (x <- plist) {
          f.print(x)
          f.print(' ')
        }
        f.println()
      }
    }
    finally {
      f.close()
    }
  }

  private def generateInstance(n : Int) : Array[Array[Int]] = {
    val a = Array.ofDim[Int](2*n, n)
    for (i <- 0 until 2*n) {
      for (j <- 0 until n) a(i)(j) = j+1
      RandomPermute.permute(a(i))
    }
    a
  }

}

class ArgsError extends Throwable("Wrong number of arguments")
