package algos.introduction

import scala.io.Source

object StableMatchingVerify {

  def main(args : Array[String]) {
    try {
      if (args.length != 2) throw new ArgsError
      val instance = readInstance(args(0))
      val solution = readSolution(args(1))
      verifySolution(instance, solution)
    }
    catch {
      case e : Throwable => {
        println("ERROR: " + e.getMessage())
        usage()
        sys.exit(-1)
      }
    }
  }

  private def usage() {
    println("USAGE: scala StableMatchingVerify.jar <input file> <output file>");
  }

  private def readInstance(fileName : String) : Array[Array[Int]] = {
    def parsePrefList(line : String) : Array[Int] =
      line.trim().split("\\s+").map(num => num.toInt - 1)
    val f = Source.fromFile(fileName)
    try {
      f.getLines.map(parsePrefList).toArray
    }
    finally {
      f.close()
    }
  }

  private def readSolution(fileName : String) : Array[(Int, Int)] = {
    def parsePair(line : String) : (Int, Int) = {
      val nums = line.trim().split("\\s+").map(num => num.toInt - 1)
      (nums(0), nums(1))
    }
    val f = Source.fromFile(fileName)
    try {
      f.getLines.map(parsePair).toArray
    }
    finally {
      f.close()
    }
  }

  private def verifySolution(instance : Array[Array[Int]], solution : Array[(Int, Int)]) {
    val n         = instance.length / 2
    val mPrefList = instance.take(n)
    val wPrefList = instance.drop(n)
    val mPartner  = Array.ofDim[Int](n)
    val wPartner  = Array.ofDim[Int](n)
    val mPrefers  = Array.ofDim[Boolean](n, n)
    val wPrefers  = Array.ofDim[Boolean](n, n)
    for ((m,w) <- solution) {
      mPartner(m) = w
      wPartner(w) = m
    }
    for (m <- 0 until n)
      for (w <- mPrefList(m).takeWhile(_ != mPartner(m)))
        mPrefers(m)(w) = true
    for (w <- 0 until n)
      for (m <- wPrefList(w).takeWhile(_ != wPartner(w)))
        wPrefers(w)(m) = true
    for (m <- 0 until n)
      for (w <- 0 until n)
        if (mPrefers(m)(w) && wPrefers(w)(m)) {
          println("Matching is not stable")
          println()
          println(s"Man ${m+1} prefers woman ${w+1} over woman ${mPartner(m)+1}")
          println(s"Woman ${w+1} prefers man ${m+1} over man ${wPartner(w)+1}")
          return
        }
    println("OK")
  }

}

class ArgsError extends Throwable("Wrong number of arguments")
